@@ -1,3 +1,5 @@
+from inspect import signature
+from types import MappingProxyType
 from re import split, sub
 from typing import DefaultDict, Dict, Union, TextIO, List, Any, Generator, Tuple
 from sys import argv
@@ -8,7 +10,7 @@ from io import StringIO, BytesIO
 from collections import defaultdict
 from pathlib import Path
 
-__version__ = '2.1.4'
+__version__ = '2.1.7'
 GLOBALS_KEY = '_globals'
 
 if os_name == 'nt':
@@ -81,7 +83,7 @@ class DictConfig(dict):
     }
 
     def __init__(self, *args, no_globals: bool = False, no_key_error: bool = False, skip_lists: bool = False,
-                 no_compound_keys: bool = False, skip_iterables: bool = False,):
+                 no_compound_keys: bool = False, skip_iterables: bool = False):
         """
         Constructor method
         """
@@ -92,7 +94,8 @@ class DictConfig(dict):
             self.globals = None
         elif no_globals:
             if isinstance(no_globals, dict):
-                self.globals = no_globals
+                # any dict type will be cast to type of self, with default settings
+                self.globals = self.__class__(no_globals)
             else:
                 self.globals = None
         else:
@@ -100,7 +103,8 @@ class DictConfig(dict):
             if GLOBALS_KEY not in self or not isinstance(super(DictConfig, self).__getitem__(GLOBALS_KEY), dict):
                 self.globals = None
             else:
-                self.globals = super(DictConfig, self).__getitem__(GLOBALS_KEY)
+                # cast _globals dict in self to self type
+                self.globals = self.__class__(super(DictConfig, self).__getitem__(GLOBALS_KEY))
                 del self[GLOBALS_KEY]
         self.filename = None
         self.arguments = None
@@ -108,6 +112,8 @@ class DictConfig(dict):
         self.cfg_filename = None
         self.parameters = None
         self.from_arguments = []
+        # this attribute is only for the benefit of the Config subclass, which exposes it as a property
+        self._shadow_attrs = False
         # replace dicts in Config with equivalent Config
         self._dicts_to_config(self, skip_iterables=skip_lists or skip_iterables)
 
@@ -194,6 +200,24 @@ class DictConfig(dict):
             return False
         return (len(keys) == 1) or (keys[1:] in self[keys[0]])
 
+    def __or__(self, other):
+        if not isinstance(other, dict):
+            return NotImplemented
+        new = self.__class__(self)
+        new.update(other)
+        return new
+
+    def __ror__(self, other):
+        if not isinstance(other, dict):
+            return NotImplemented
+        new = self.__class__(other)
+        new.update(self)
+        return new
+
+    def __ior__(self, other):
+        self.__class__.update(self, other)
+        return self
+
     def _get_direct(self, key: str) -> Any:
         """
         Retrieve the item with (simple only) key from self and without performing global substitutions
@@ -223,8 +247,12 @@ class DictConfig(dict):
                 # noinspection PyArgumentList
                 return value.__class__(self._subst_globals(v) for v in value)
             elif isinstance(value, dict):
-                # noinspection PyArgumentList
-                return value.__class__({k: self._subst_globals(v) for k, v in value.items()})
+                if isinstance(value, Config):
+                    # return the config at this time, as it will perform replacements when needed (if needed)
+                    return value
+                else:
+                    # noinspection PyArgumentList
+                    return value.__class__({k: self._subst_globals(v) for k, v in value.items()})
             else:
                 return value
 
@@ -854,6 +882,14 @@ class Config(DictConfig):
     >>> print(dc['foo'])
     'qux'
     """
+    @property
+    def shadow_attrs(self):
+        return self._shadow_attrs
+
+    @shadow_attrs.setter
+    def shadow_attrs(self, value):
+        self._shadow_attrs = value
+
     def __getattr__(self, attr):
         if attr in self:
             return self[attr]
@@ -861,8 +897,8 @@ class Config(DictConfig):
             raise AttributeError(f'No attribute or key {attr} for {self.__class__}')
 
     def __setattr__(self, attr, value):
-        # don't allow configuration items to shadow object attributes, not using hasattr, as it will call __getattr__
-        if attr in dir(self) or attr not in self:
+        # check if configuration items can shadow object attributes, not using hasattr, as it will call __getattr__
+        if (attr in dir(self) or attr not in self) and (not hasattr(self, '_shadow_attrs') or not self._shadow_attrs):
             super(DictConfig, self).__setattr__(attr, value)
         else:
             self[attr] = value
