@@ -75,12 +75,36 @@ class TestConfig(unittest.TestCase):
         self.assertEqual('1={x}', cfg['test'][1], msg='escaped braces in lists should be unescaped')
         self.assertFalse('_globals' in cfg, msg='globals should be hidden')
 
+    def test_globals_list_config(self):
+        cfg = DictConfig({'_globals': {'x': 1}, 'a': {'b': '{x}'}, 'c': [{'d': '{x}'}]})
+        cfg_a = cfg['a']
+        self.assertFalse('_globals' in cfg_a, msg='globals should be hidden for extracted child')
+        self.assertEqual('1', cfg_a['b'], msg='globals should be propagated to Config child')
+        cfg_list = cfg['c']
+        self.assertEqual('1', cfg_list[0]['d'], msg='globals should be propagated to Config children in lists')
+
     def test_globals_noglobals(self):
         cfg = DictConfig({'_globals': {'x': 1}, 'test': '1={x}', 'escaped': '1={{x}}'}, no_globals=True)
         self.assertEqual('1={x}', cfg['test'], msg='noglobals, globals should not be replaced')
         self.assertEqual('1={{x}}', cfg['escaped'], msg='noglobals, escaped braces should not be unescaped')
         self.assertTrue('_globals' in cfg, msg='noglobals, globals should be visible')
 
+    def test_globals_as_config(self):
+        cfg = Config({'_globals': {'x': {'y': 1}}, 'a': {'b': 1}} )
+        self.assertEqual(1, cfg.globals.x.y, msg='globals function as a config')
+        a = cfg.a
+        self.assertEqual(1, a.globals.x.y, msg='globals function as a config on a copy')
+        self.assertEqual({'y': 1}, a.globals.x, msg='globals compare as a dict')
+
+    def test_shadow_attrs(self):
+        cfg = Config()
+        cfg.shadow_attrs = True
+        cfg['a'] = 1
+        cfg.a = 2
+        cfg.b = 1
+        cfg['b'] = 2
+        self.assertEqual((2, 2, 2, 2), (cfg.a, cfg['a'], cfg.b, cfg['b']), msg='attributes can be shadowed')
+
     def test_key_error(self):
         cfg = DictConfig({'test': 1})
         with self.assertRaises(KeyError, msg='without no_key_error, reading non-existent keys raises an exception'):
@@ -151,6 +175,19 @@ class TestConfig(unittest.TestCase):
         del cfg.test
         self.assertEqual(2, cfg.test, 'if no longer shadowed by an attribute, keys can be access as attribute')
 
+    def test_update(self):
+        cfg = Config({1: 'a'})
+        cfg = cfg | Config({1: 'b', 2: 'c'})
+        self.assertEqual(('b', 'c'), (cfg[1], cfg[2]), 'updated values are correct')
+        self.assertIsInstance(cfg, Config, msg='update should not affect type')
+
+        cfg = DictConfig({1: 'a'})
+        cfg = cfg | Config({1: 'b', 2: 'c'})
+        self.assertIsInstance(cfg, DictConfig, msg='update with different Config type should not affect type')
+
+        cfg = cfg | {1: 'b', 2: 'c'}
+        self.assertIsInstance(cfg, DictConfig, msg='update with dict should not affect type')
+
 
 if __name__ == '__main__':
     unittest.main()
